// ----------------------------------------------------------------------------
//
// rt++.h
// RT++ --- higher order threads for C++
//
// $Id: rt++.h,v 1.32 1996/04/03 08:32:49 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_H
#define RT_H

#define RT_VERSION "1.0"
 
// programs may define the following flags before including this file; 
//
// #define RT_STACK_CHECK: check stack overflow on each variable push
// #define RT_THREAD_NOTPAR: threads are sequentially executed
// #define RT_THREAD_NOTLAZY: default is creation of active threads
// #define RT_TBAG_NOTLAZY: default is creation of active bag threads

// ----------------------------------------------------------------------------
//
// class definitions and function prototypes
//
// ----------------------------------------------------------------------------

#include "rt++/rt_machine.h" // machine-specific interface
#include "rt++/rt_mutex.h"   // mutual exclusion objects 
#include "rt++/rt_stack.h"   // stack information

#include "rt++/rt_shared.h"  // shared memory system
#include "rt++/rt_block.h"   // heap block layout
#include "rt++/rt_heap.h"    // garbage-collected heap

#include "rt++/rt_thread.h"  // interface to system threads
#include "rt++/rt_core.h"    // core threads 
#include "rt++/rt_proc.h"    // processor-global data
#include "rt++/rt_system.h"  // system-global data

#include "rt++/rt_type.h"    // type information

#include "rt++/rt_array.h"   // garbage-collected arrays
#include "rt++/rt_pointer.h" // garbage-collected pointer objects
#include "rt++/rt_list.h"    // garbage-collected lists

#include "rt++/rt_result.h"  // result threads 
#include "rt++/rt_arg.h"     // argument threads 
#include "rt++/rt_tptr.h"    // thread pointers
#include "rt++/rt_tbag.h"    // thread bags

#include "rt++/rt_stat.h"    // statistics information
#include "rt++/rt_lib.h"     // library routines
#include "rt++/rt_main.h"    // main functions

// ----------------------------------------------------------------------------
//
// inline code
//
// ----------------------------------------------------------------------------

#include "rt++/rt_general.i" // general functions
#include "rt++/rt_mutex.i"   // mutual exclusion objects 
#include "rt++/rt_stack.i"   // stack information

#include "rt++/rt_shared.i"  // shared memory system
#include "rt++/rt_block.i"   // heap block layout
#include "rt++/rt_heap.i"    // garbage-collected heap

#include "rt++/rt_thread.i"  // interface to system threads
#include "rt++/rt_core.i"    // core threads 
#include "rt++/rt_proc.i"    // processor-global data

#include "rt++/rt_array.i"   // garbage-collected arrays
#include "rt++/rt_pointer.i" // garbage-collected pointer objects
#include "rt++/rt_list.i"    // garbage-collected lists

#include "rt++/rt_result.i"  // result threads 
#include "rt++/rt_arg.i"     // argument threads 
#include "rt++/rt_tptr.i"    // thread pointers
#include "rt++/rt_tbag.i"    // thread bags

#endif
// ----------------------------------------------------------------------------
// end of rt++.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
