// ----------------------------------------------------------------------------
//
// rt_arg.i
// RT++ argument threads (fully typed)
//
// $Id: rt_arg.i,v 1.3 1996/03/22 08:46:01 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_ARG_I
#define RT_ARG_I

// ----------------------------------------------------------------------------
//
// thread with zero arguments
//
// ----------------------------------------------------------------------------

template <class R>
inline void 
ArgumentThread0<R>::user(void) 
{
  result = f();
}

template <class R>
inline void
ArgumentThread0<R>::scan(void)
{
  ResultThread<R>::_scan();
}

template <class R>
inline void 
zero(void)
{
}

template <class R>
inline void 
ArgumentThread0<R>::init(R (*_f)(void)) 
{
  ResultThread<R>::init();
  f = _f;
  userfun = (void(*)(CoreThread*))user;
  scanfun = (void(*)(CoreThread*))ArgumentThread0<R>::scan;
  // instance not generated by g++
  // zerofun = (void(*)(CoreThread*))zero;
}

template <class R>
inline 
ArgumentThread0<R>::ArgumentThread0(R (*_f)(void))
{
  init(_f);
}

// ----------------------------------------------------------------------------
//
// thread with one argument
//
// ----------------------------------------------------------------------------

template <class R, class A1>
inline void 
ArgumentThread1<R, A1>::user(void) 
{
  result = f(a1);
}

template <class R, class A1>
inline void 
ArgumentThread1<R, A1>::scan(void)
{
  ResultThread<R>::_scan();
  void(*const f1)(A1*) = Type<A1>::scan;
  if (f1) f1(&a1);
}

template <class R, class A1>
inline void 
ArgumentThread1<R, A1>::zero(void)
{
  void(*const f1)(A1*) = Type<A1>::init;
  if (f1) f1(&a1);
}

template <class R, class A1>
inline void 
ArgumentThread1<R, A1>::init(R (*_f)(A1), A1 _a1) 
{
  ResultThread<R>::init();
  f = _f;
  a1 = _a1;
  userfun = (void(*)(CoreThread*))user;
  scanfun = (void(*)(CoreThread*))ArgumentThread1<R, A1>::scan;
  zerofun = (void(*)(CoreThread*))zero;
}

template <class R, class A1>
inline 
ArgumentThread1<R, A1>::ArgumentThread1(R (*_f)(A1), A1 _a1)
{
  init(_f, _a1);
}

// ----------------------------------------------------------------------------
//
// thread with two arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2>
inline void 
ArgumentThread2<R, A1, A2>::user(void) 
{
  result = f(a1, a2);
}

template <class R, class A1, class A2>
inline void 
ArgumentThread2<R, A1, A2>::scan(void)
{
  ResultThread<R>::_scan();
  void(*const f1)(A1*) = Type<A1>::scan;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::scan;
  if (f2) f2(&a2);
}

template <class R, class A1, class A2>
inline void 
ArgumentThread2<R, A1, A2>::zero(void)
{
  void(*const f1)(A1*) = Type<A1>::init;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::init;
  if (f2) f2(&a2);
}

template <class R, class A1, class A2>
inline void 
ArgumentThread2<R, A1, A2>::init(R (*_f)(A1, A2), A1 _a1, A2 _a2)
{
  ResultThread<R>::init();
  f = _f;
  a1 = _a1;
  a2 = _a2;
  userfun = (void(*)(CoreThread*))user;
  scanfun = (void(*)(CoreThread*))ArgumentThread2<R, A1, A2>::scan;
  zerofun = (void(*)(CoreThread*))zero;
}

template <class R, class A1, class A2>
inline 
ArgumentThread2<R, A1, A2>::ArgumentThread2(R (*_f)(A1, A2), A1 _a1, A2 _a2)
{
  init(_f, _a1, _a2);
}

// ----------------------------------------------------------------------------
//
// thread with three arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2, class A3>
inline void 
ArgumentThread3<R, A1, A2, A3>::user(void) 
{
  result = f(a1, a2, a3);
}

template <class R, class A1, class A2, class A3>
inline void 
ArgumentThread3<R, A1, A2, A3>::scan(void)
{
  ResultThread<R>::_scan();
  void(*const f1)(A1*) = Type<A1>::scan;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::scan;
  if (f2) f2(&a2);
  void(*const f3)(A3*) = Type<A3>::scan;
  if (f3) f3(&a3);
}

template <class R, class A1, class A2, class A3>
inline void 
ArgumentThread3<R, A1, A2, A3>::zero(void)
{
  void(*const f1)(A1*) = Type<A1>::init;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::init;
  if (f2) f2(&a2);
  void(*const f3)(A3*) = Type<A3>::init;
  if (f3) f3(&a3);
}

template <class R, class A1, class A2, class A3>
inline void 
ArgumentThread3<R, A1, A2, A3>::
init(R (*_f)(A1, A2, A3), A1 _a1, A2 _a2, A3 _a3)
{
  ResultThread<R>::init();
  f = _f;
  a1 = _a1;
  a2 = _a2;
  a3 = _a3;
  userfun = (void(*)(CoreThread*))user;
  scanfun = (void(*)(CoreThread*))ArgumentThread3<R, A1, A2, A3>::scan;
  zerofun = (void(*)(CoreThread*))zero;
}

template <class R, class A1, class A2, class A3>
inline 
ArgumentThread3<R, A1, A2, A3>::
ArgumentThread3(R (*_f)(A1, A2, A3), A1 _a1, A2 _a2, A3 _a3)
{
  init(_f, _a1, _a2, _a3);
}

// ----------------------------------------------------------------------------
//
// thread with four arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2, class A3, class A4>
inline void 
ArgumentThread4<R, A1, A2, A3, A4>::user(void) 
{
  result = f(a1, a2, a3, a4);
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
ArgumentThread4<R, A1, A2, A3, A4>::scan(void)
{
  ResultThread<R>::_scan();
  void(*const f1)(A1*) = Type<A1>::scan;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::scan;
  if (f2) f2(&a2);
  void(*const f3)(A3*) = Type<A3>::scan;
  if (f3) f3(&a3);
  void(*const f4)(A4*) = Type<A4>::scan;
  if (f4) f4(&a4);
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
ArgumentThread4<R, A1, A2, A3, A4>::zero(void)
{
  void(*const f1)(A1*) = Type<A1>::init;
  if (f1) f1(&a1);
  void(*const f2)(A2*) = Type<A2>::init;
  if (f2) f2(&a2);
  void(*const f3)(A3*) = Type<A3>::init;
  if (f3) f3(&a3);
  void(*const f4)(A4*) = Type<A4>::init;
  if (f4) f4(&a4);      
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
ArgumentThread4<R, A1, A2, A3, A4>::
init(R (*_f)(A1, A2, A3, A4), A1 _a1, A2 _a2, A3 _a3, A4 _a4)
{
  ResultThread<R>::init();
  f = _f;
  a1 = _a1;
  a2 = _a2;
  a3 = _a3;
  a4 = _a4;
  userfun = (void(*)(CoreThread*))user;
  scanfun = (void(*)(CoreThread*))ArgumentThread4<R, A1, A2, A3, A4>::scan;
  zerofun = (void(*)(CoreThread*))zero;
}

template <class R, class A1, class A2, class A3, class A4>
inline 
ArgumentThread4<R, A1, A2, A3, A4>::
ArgumentThread4(R (*_f)(A1, A2, A3, A4), A1 _a1, A2 _a2, A3 _a3, A4 _a4)
{
  init(_f, _a1, _a2, _a3, _a4);
}

#endif
// ----------------------------------------------------------------------------
// end of rt_arg.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
