// ----------------------------------------------------------------------------
//
// rt_array.i
// RT++ garbage collected arrays
//
// $Id: rt_array.i,v 1.4 1996/04/03 08:32:51 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_ARRAY_I
#define RT_ARRAY_I

// ----------------------------------------------------------------------------
//
// return block reference from pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline ArrayBlock *
Array<T>::block(T *p)
{
  void *q = rt_aligndown((void*)p-sizeof(ArrayInfo));
  return((ArrayBlock*)Block::block(q));   
}

// ----------------------------------------------------------------------------
//
// initialize pointer to array
//
// ----------------------------------------------------------------------------
template <class T>
inline
Array<T>::Array(T *p)
{
  pointer = p;
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// array iniitialization
// caller is in charge of unlocking the processor!!!
//
// ----------------------------------------------------------------------------
template <class T>
inline void
Array<T>::_alloc(int n)
{
  int size = Block::align((int)rt_align((void*)sizeof(int))+n*sizeof(T));
  int *addr = (int*)processor.alloc(size);
  *addr = n;
  pointer = (T*)(rt_align(addr+1));
  void (*const f)(T *) = Type<T>::init; 
  if (f)
    {
      T *p = pointer;
      T *end = p+n;
      while (p < end)
	{
	  f(p);
	  p++;
	}
    }
}

// ----------------------------------------------------------------------------
//
// nil iniitialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void
Array<T>::init(void)
{
  pointer = 0;
}

// ----------------------------------------------------------------------------
//
// array iniitialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void
Array<T>::alloc(int n)
{
  _alloc(n);
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by nil array
//
// ----------------------------------------------------------------------------
template <class T>
inline
Array<T>::Array(void)
{
  init();
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// allocate array of <n> objects
//
// ----------------------------------------------------------------------------
template <class T>
inline
Array<T>::Array(int n)
{
  _alloc(n);
  processor.push(this, (void(*)(void*))scan);
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by array <a>
//
// ----------------------------------------------------------------------------
template <class T>
inline
Array<T>::Array(const Array<T> &a)
{
  pointer = a.pointer;
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// destroy array reference
//
// ----------------------------------------------------------------------------
template <class T>
inline
Array<T>::~Array(void)
{
  processor.pop(&pointer);
}

// ----------------------------------------------------------------------------
//
// assignment of arrays
//
// ----------------------------------------------------------------------------
template <class T>
inline Array<T>& 
Array<T>::operator = (const Array<T> &a)
{
  pointer = a.pointer;
  return(*this);
}

// ----------------------------------------------------------------------------
//
// reference equality of arrays
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Array<T>::operator == (const Array<T> &a)
{
  return(pointer == a.pointer);
}

// ----------------------------------------------------------------------------
//
// test for nil array (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Null(const Array<T> &a)
{
  return(a.pointer == 0);
}

// ----------------------------------------------------------------------------
//
// allocate array of <n> objects (static member function)
//
// ----------------------------------------------------------------------------
template <class T>
inline Array<T> 
Array<T>::Alloc(int n)
{
  Array<T> p(n);
  return(p);
}

// ----------------------------------------------------------------------------
//
// return number of array elements (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Length(Array<T> a)
{
  return(Array<T>::block(a.pointer)->info.length);   
}

// ----------------------------------------------------------------------------
//
// indexing of arrays, returns an alias!
//
// ----------------------------------------------------------------------------
template <class T>
inline T& 
Array<T>::operator [] (int i)
{
  return(pointer[i]);
}

// ----------------------------------------------------------------------------
//
// scan referenced array
//
// ----------------------------------------------------------------------------
template <class T>
inline void
Array<T>::scan(void)
{
  T *p = pointer;
  if (!p) return;               
  if (block(p)->mark()) return;
  void (*const f)(T *) = Type<T>::scan; 
  if (!f) return;
  T *l = p+block(p)->info.length;
  while (p < l)   
    {
      f(p);
      p++;        
    }
}

#endif
// ----------------------------------------------------------------------------
// end of rt_array.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
