// ----------------------------------------------------------------------------
//
// rt_block.i
// RT++ heap block layout
//
// $Id: rt_block.i,v 1.4 1996/04/03 08:32:53 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_BLOCK_I
#define RT_BLOCK_I

// ----------------------------------------------------------------------------
// 
// mark block; return 1 if already marked
//
// ----------------------------------------------------------------------------
inline int 
Block::mark(void)
{
  if (size & 1)
    {
      return(1);
    }
  else
    {
      size |= 1;
      return(0);
    }
}

// ----------------------------------------------------------------------------
// 
// unmark block; return 0 if not marked
//
// ----------------------------------------------------------------------------
inline int 
Block::unmark(void)
{
  if (size & 1)
    {
      size &= ~1;
      return(1);
    }
  else
    {
      return(0);
    }
}

// ----------------------------------------------------------------------------
// 
// return 1 iff block is marked
//
// ----------------------------------------------------------------------------
inline int 
Block::is_marked(void)
{
  return (size & 1);
}

// ----------------------------------------------------------------------------
// 
// return block header information from subsequent memory address
//
// ----------------------------------------------------------------------------
inline Block *
Block::block(void *p)
{
  return(((Block*)p)-1);
}

// ----------------------------------------------------------------------------
//
// return block alignment
// 
// ----------------------------------------------------------------------------
inline int 
Block::alignment(void)
{
  return(sizeof(FreeBlock));
}

// ----------------------------------------------------------------------------
//
// return aligned memory block size from size <s>
// this assumes that alignment() is a power of two!!!
//
// ----------------------------------------------------------------------------
inline int
Block::align(int s)
{
  const unsigned long int a = alignment()-1;
  return (((unsigned long int)(s) + a) & ~a);
}

#endif
// ----------------------------------------------------------------------------
// end of rt_block.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
