// ----------------------------------------------------------------------------
//
// rt_core.h
// RT++ core threads (untyped)
//
// $Id: rt_core.h,v 1.26 1996/03/17 16:00:40 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_CORE_H
#define RT_CORE_H

// ----------------------------------------------------------------------------
//
// untyped core threads
//
// ----------------------------------------------------------------------------
class CoreThread: public SystemThread
{
  private:
  Stack shadow;                      // shadow stack
  Mutex mutex;                       // synchronization on waiters 

  int executing;                     // is thread already executing?
  int stopped;                       // is executing thread stopped?
  int bound;                         // proc to which thread is bound or -1
  CoreThread *waiters;               // list of blocked threads

  protected:                         // flag used in ResultThread::wait()
  int avail;                         // is result available? 
  void _stop(void);                  // kernel of stop for use by thread bags

  public:
  CoreThread *next;                  // link to next task in ready/lazy queue
  CoreThread *prev;                  // link to previous task in lazy queue
  CoreThread *lazy;                  // stack of executed lazy threads
  CoreThread *root_next;             // next root of gc (if in root set)
  CoreThread *root_prev;             // previous root of gc (if in root set)

  // userfun and scanfun must be redefined by subclass
  // zerofun may be redefined by subclass
  void (*userfun)(CoreThread *);     // user function executed by shell
  void (*scanfun)(CoreThread*);      // scan function executed by gc
  void (*zerofun)(CoreThread *);     // zero function executed on termination

  void init(void);                   // initialize core thread
  void scan(void);                   // scan thread for garbage collection
  CoreThread(void);                  // create core thread (unused)

  void lock(void);                   // lock thread
  void unlock(void);                 // unlock thread

  int is_stopped(void);              // return true iff thread is stopped
  void check(void);                  // check for stack overflow
  Stack *get_shadow(void);           // return shadow stack
  void set_shadow(Stack *s);         // set shadow stack

  void realize(void);                // realize thread
  void wait(void);                   // wait until thread result available
  void deliver(void);                // deliver result to waiting threads
  void stop(void);                   // prevent (further) execution of thread
  int execute(void);                 // mark thread as being executed

  void bind(int);                    // bind thread to specific processor
  void unbind(void);                 // unbind thread
  int binding(void);                 // return binding

  void root(void);                   // declare thread as root of gc
  void noroot(void);                 // reset declaration
};

#endif
// ----------------------------------------------------------------------------
// end of rt_core.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
