// ----------------------------------------------------------------------------
//
// rt_core.i
// RT++ core threads (untyped)
//
// $Id: rt_core.i,v 1.3 1996/02/28 20:15:11 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_CORE_I
#define RT_CORE_I

// ----------------------------------------------------------------------------
//
// initialize core thread
//
// ----------------------------------------------------------------------------
inline void
CoreThread::init(void)
{
  SystemThread::init();
  shadow.init();
  mutex.init();
  executing = 0;
  stopped = 0;
  bound = -1;
  waiters = 0;
  avail = 0;
  lazy = 0;
  zerofun = 0;
}

// ----------------------------------------------------------------------------
//
// scan thread for garbage collection
//
// ----------------------------------------------------------------------------
inline void
CoreThread::scan(void)
{
  if (!shadow.null()) heap->push(&shadow);
  CoreThread *thread;
  thread = waiters;
  while (thread)
    {
      if (!Block::block(thread)->mark())
	{
	  thread->scanfun(thread);
	}
      thread = thread->next;
    }
  thread = lazy;
  while (thread)
    {
      if (!Block::block(thread)->mark())
	{
	  thread->scanfun(thread);
	}
      thread = thread->next;
    }
}

// ----------------------------------------------------------------------------
//
// create a new core thread
//
// ----------------------------------------------------------------------------
inline
CoreThread::CoreThread(void)
{ 
  init();
}

// ----------------------------------------------------------------------------
//
// lock core thread
//
// ----------------------------------------------------------------------------
inline void 
CoreThread::lock(void)
{
  mutex.lock();
}

// ----------------------------------------------------------------------------
//
// unlock core thread
//
// ----------------------------------------------------------------------------
inline void 
CoreThread::unlock(void) 
{
  mutex.unlock();
}

// ----------------------------------------------------------------------------
//
// return true iff thread is stopped
//
// ----------------------------------------------------------------------------
inline int
CoreThread::is_stopped(void)
{
  return(stopped);
}

// ----------------------------------------------------------------------------
//
// abort program if thread is close to end of stack
//
// ----------------------------------------------------------------------------
inline void
CoreThread::check(void)
{
  shadow.check();
}

// ----------------------------------------------------------------------------
//
// return shadow stack
//
// ----------------------------------------------------------------------------
inline Stack *
CoreThread::get_shadow(void)
{
  return(&shadow);
}

// ----------------------------------------------------------------------------
//
// set shadow stack
//
// ----------------------------------------------------------------------------
inline void 
CoreThread::set_shadow(Stack *s)
{
  shadow = *s;
}

// ----------------------------------------------------------------------------
//
// return thread binding
//
// ----------------------------------------------------------------------------
inline int
CoreThread::binding(void)
{
  return(bound);
}

#endif
// ----------------------------------------------------------------------------
// end of rt_core.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
