// ----------------------------------------------------------------------------
//
// rt_heap.h
// RT++ garbage collected heap
//
// $Id: rt_heap.h,v 1.18 1996/03/16 15:17:22 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_HEAP_H
#define RT_HEAP_H

class CoreThread;

// ----------------------------------------------------------------------------
// 
// garbage collected heap
// the heap is organized in clusters (initially 1, later extended)
// all clusters are of the same size
// each cluster in organized in blocks (initially blocknum, later scattered)
// the initial blocks are never combined (distribution among processors)
//
// ----------------------------------------------------------------------------
class Heap
{
  int size;                   // size of cluster in bytes
  int blocknum;               // initial number of blocks in cluster
  int blocksize;              // initial size of blocks in cluster
  FreeBlock *start;           // pointer to first cluster
  FreeBlock *free;            // first free block in heap
  Mutex mutex_alloc;          // mutual exclusion for allocation
  Mutex mutex_gc;             // mutual exclusion for garbage collection
  volatile int triggered;     // set iff gc is triggered
  int trigger;                // number of trigger processor
  int triggersize;            // size of request that triggered gc
  int maxsize;                // size of largest block after gc
  volatile int notready;      // number of not yet ready processors
  volatile int markers;       // number of currently marking processors
  volatile int _markers;      // number of currently marking processors
  volatile int sweepers;      // number of currently sweeping processors
  FreeBlock *clusters;        // cluster list for garbage collection
  Block *blocks;              // next block in cluster for garbage collection
  Block *clusterend;          // end of cluster for garbage collection
  CoreThread *threads;        // root threads for garbage collection
  volatile Stack *stacks;     // stack list for mark phase
  int freesize;               // size of free heap after gc
  int freeblocks;             // number of free blocks after gc
  FreeBlock *cluster(FreeBlock *free); // allocate new cluster, link with free
  Block *search(int s);       // return memory block of at least s bytes
  void mark(void);            // mark phase of garbage collection
  void sweep(void);           // sweep phase of garbage collection
  void gc_inc(void);          // increase heap by new cluster
  public:
  Heap(int size, int blocks); // init heap with <size> bytes in <blocks>
  int heapsize(void);         // return heap size in bytes
  int blocknumber(void);      // return number of heap blocks
  FreeBlock *alloc(int s, FreeBlock **prev); // return free block list
  void gc(void);              // perform garbage collection
  void push(Stack*);          // link thread stack into list for gc
  void collect(void);         // trigger garbage collection
};

// heap object, dynamically allocated in shared memory
extern Heap *heap;

// initialize heap with at least <size> bytes partitioned into <blocks>
void heap_init(int size, int blocks);

#endif
// ----------------------------------------------------------------------------
// end of rt_heap.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
