// ----------------------------------------------------------------------------
//
// rt_list.i
// RT++ garbage collected lists
//
// $Id: rt_list.i,v 1.2 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_LIST_I
#define RT_LIST_I

// ----------------------------------------------------------------------------
//
// return block reference from pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline Block *
List<T>::block(ListNode<T> *p)
{
  return((Block*)Block::block(p));   
}

// ----------------------------------------------------------------------------
//
// initialize pointer to object
//
// ----------------------------------------------------------------------------
template <class T>
inline
List<T>::List(ListNode<T> *p)
{
  pointer = p;
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// list node initialization
// caller is in charge of unlocking processor!!!
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::_cons(const T &_head, const List<T> &_tail)
{
  ListNode<T> *l = (ListNode<T>*)processor.alloc(sizeof(ListNode<T>));
  l->head = _head;
  l->tail = _tail.pointer;
  pointer = l;
}

// ----------------------------------------------------------------------------
//
// nil initialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::init(void)
{
  pointer = 0;
}

// ----------------------------------------------------------------------------
//
// nil initialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::nil(void)
{
  init();
}

// ----------------------------------------------------------------------------
//
// list node initialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::cons(const T &_head, const List<T> &_tail)
{
  _cons(_head, _tail);
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by nil pointer
//
// ----------------------------------------------------------------------------
template <class T>
inline
List<T>::List(void)
{
  init();
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// initialization by list node
//
// ----------------------------------------------------------------------------
template <class T>
inline
List<T>::List(const T &_head, const List<T> &_tail)
{
  cons(_head, _tail);
  processor.push(this, (void(*)(void*))scan);
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline
List<T>::List(const List<T> &l)
{
  pointer = l.pointer;
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// destroy list reference
//
// ----------------------------------------------------------------------------
template <class T>
inline
List<T>::~List(void)
{
  processor.pop(&pointer);
}

// ----------------------------------------------------------------------------
//
// assignment of list <l>
//
// ----------------------------------------------------------------------------
template <class T> 
inline List<T>& 
List<T>::operator = (const List<T> &l)
{
  pointer = l.pointer;
  return(*this);
}

// ----------------------------------------------------------------------------
//
// equality of list references
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
List<T>::operator == (const List<T> &l)
{
  return(pointer == l.pointer);
}

// ----------------------------------------------------------------------------
//
// test for nil list (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Null(const List<T> &l)
{
  return(l.pointer == 0);
}

// ----------------------------------------------------------------------------
//
// construction of a nil list
//
// ----------------------------------------------------------------------------
template <class T>
inline List<T> 
List<T>::Nil(void)
{
  List<T> l;
  return(l);
}

// ----------------------------------------------------------------------------
//
// construction of a list node (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline List<T> 
Cons(const T &_head, const List<T> &_tail)
{
  List<T> list(_head, _tail);
  return(list);
}

// ----------------------------------------------------------------------------
//
// return head element (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline T 
Head(const List<T> &l)
{
  return(l.pointer->head);
}

// ----------------------------------------------------------------------------
//
// set head element
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::head(const T &_head)
{
  pointer->head = _head;
}

// ----------------------------------------------------------------------------
//
// return tail list (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline List<T> 
Tail(const List<T> &l)
{
  return(List<T>(l.pointer->tail));
}

// ----------------------------------------------------------------------------
//
// set tail element
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::tail(const List<T> &_tail)
{
  pointer->tail = _tail.pointer;
}

// ----------------------------------------------------------------------------
//
// set reference to tail
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::tail(void)
{
  pointer = pointer->tail;
}

// ----------------------------------------------------------------------------
//
// scan referenced list
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
List<T>::scan (void)
{
  void(*const f)(T*) = Type<T>::scan;
  if (f)
    {
      // pointer.scan();
      ListNode<T> *p = pointer;
      while (p)
	{
	  if (block(p)->mark()) return;
	  f(&p->head);
	  p = p->tail;
	}
    }
  else
    {
      // pointer.scan();
      ListNode<T> *p = pointer;
      while (p)
	{
	  if (block(p)->mark()) return;
	  p = p->tail;
	}
    }
}

#endif
// ----------------------------------------------------------------------------
// end of rt_list.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
