// ----------------------------------------------------------------------------
//
// rt_machine.h
// RT++ machine-specific interface
//
// $Id: rt_machine.h,v 1.32 1996/04/03 15:43:01 ws Exp ws $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_MACHINE_H
#define RT_MACHINE_H

// ----------------------------------------------------------------------------
//
// user configurations
//
// ----------------------------------------------------------------------------

// operating system: define one of the following
// RT_LINUX   ... Linux
// RT_PTX     ... Sequent Dynix/ptx
// RT_IRIX    ... Silicon Graphics IRIX
// RT_SUNOS   ... Sun SunOS 4.1.3
// RT_SOLARIS ... Sun Solaris 2.x
// RT_HPUX    ... Hewlett Packard HP/UX
// RT_CONVEX  ... Convex SPP under HP/UX 
#define RT_CONVEX

// processor: define one of the following
// RT_i386   ... Intel 386 and compatibles
// RT_MIPS   ... MIPS R2000 or R3000
// RT_MIPS2  ... MIPS R4000 and higher
// RT_SPARC  ... SPARC
// RT_PARISC ... HP PA-RISC
#define RT_PARISC

// thread package: define one of the following ;-)
// RT_QT     ... quick threads package
#define RT_QT

// ----------------------------------------------------------------------------
//
// standard header files
//
// ----------------------------------------------------------------------------

#include <sys/types.h> // system types
#include <sys/wait.h>  // wait interface
#include <sys/times.h> // timings
#include <sys/param.h> // timings
#include <unistd.h>    // standard Unix interface
#include <signal.h>    // signal interface
#include <stdlib.h>    // standard library
#include <iostream.h>  // input/output interface
#include <string.h>    // string handling
#include <stdio.h>     // sprintf
#include <time.h>      // timings
#include <setjmp.h>    // non-local jumps
#include <limits.h>    // CLK_TCK on some systems
#include <fcntl.h>     // low-level file io

// ----------------------------------------------------------------------------
//
// threads
//
// ----------------------------------------------------------------------------

#ifdef RT_QT
#include "qt.h"        // the quick threads package
#endif

// ----------------------------------------------------------------------------
//
// shared memory
//
// ----------------------------------------------------------------------------

// core functions
void *rt_shm_alloc(int n); // called by RT++ main proc to alloc shared memory
void rt_shm_unmap(void);   // called by RT++ main proc before forking
void rt_shm_map(void);     // called by each RT++ process after forking
void rt_shm_close(void);   // called by RT++ main proc after forking
void rt_shm_free(void);    // called by RT++ main proc to free shared memory

// ----------------------------------------------------------------------------
//
// shared memory selection: define one of the following
// RT_IPC  ... System V Interprocessor Communication
// RT_MMAP ... BSD memory mapped files (mmap)
// (all system provide both but IPC is more portable)
//
// ----------------------------------------------------------------------------

// both available, only IPC tested
#ifdef RT_LINUX
#define RT_IPC
#endif

// both available and tested, IPC configuration too small on our machine
#ifdef RT_PTX
#define RT_MMAP
#endif

// both available, only IPC tested
#ifdef RT_IRIX
#define RT_IPC
#endif

// both available, only IPC tested
#ifdef RT_SUNOS
#define RT_IPC
#endif

// both available, only IPC tested
#ifdef RT_SOLARIS
#define RT_IPC
#endif

// both available, only IPC tested
#ifdef RT_HPUX
#define RT_IPC
#endif

// both available, only IPC tested
// additional experimental versions of IPC and mmap
#ifdef RT_CONVEX
#define RT_IPC
#endif

// ----------------------------------------------------------------------------
//
// shared memory standard header files
//
// ----------------------------------------------------------------------------

#ifdef RT_IPC
#include <sys/ipc.h>   // SysV inter process communication
#include <sys/shm.h>   // SysV shared memory
#endif

#ifdef RT_MMAP
#ifdef RT_CONVEX
extern "C" 
{
#include <sys/mman.h>  // memory mapped files
}
#else
#include <sys/mman.h>  // memory mapped files
#endif
#endif

// ----------------------------------------------------------------------------
//
// shared memory additional definitions
//
// ----------------------------------------------------------------------------

#ifdef RT_PTX
#ifdef RT_IPC
// Sequent does not define these
extern "C"
{
  int shmget (key_t key, int size, int shmflg);
  void *shmat(int shmid, void *shmaddr, int shmflg);
  int shmctl(int shmid, int cmd, void *buffer);
}
#endif
#endif

#ifdef RT_SUNOS
#ifdef RT_IPC
// Sun does not define these
extern "C"
{
  int shmget (key_t key, int size, int shmflg);
  void *shmat(int shmid, void *shmaddr, int shmflg);
  int shmctl(int shmid, int cmd, void *buffer);
}
#endif
#endif

#ifdef RT_CONVEX
#ifdef RT_MMAP
#include <sys/cnx_mman.h>
#endif
#endif

// ----------------------------------------------------------------------------
//
// process forking
//
// ----------------------------------------------------------------------------

// core functions
void rt_fork_start(void); // called by RT++ before processes are forked
pid_t rt_fork(void);      // called by RT++ to fork a new process
void rt_fork_end(void);   // called by RT++ after processes have been forked

// ----------------------------------------------------------------------------
//
// process forking additional definitions
//
// ----------------------------------------------------------------------------

#ifdef RT_CONVEX
#include <sys/cnx_types.h>
// Convex does not define these
extern "C" { pid_t cnx_sc_fork(cnx_scid_t scid, cnx_node_t node); }
void rt_sc_add(cnx_scid_t id, int proc); // subcomplex <id> with <proc> procs
void rt_sc_reset(void);                  // reset subcomplex declarations
#endif

// ----------------------------------------------------------------------------
//
// signal handlers
// defined SA_HANDLER (handler component of sigaction structure)
//
// ----------------------------------------------------------------------------

#ifdef RT_LINUX
#define SA_HANDLER(f) ((void (*)(int))(f))
#endif

#ifdef RT_PTX
#define SA_HANDLER(f) ((void (*)(...))(f))
#endif

#ifdef RT_IRIX
#define SA_HANDLER(f) ((void (*)(...))(f))
#endif

#ifdef RT_SUNOS
#define SA_HANDLER(f) ((void (*)(...))(f))
// not provided by SunOS
static inline int raise(int sig) 
{
  return(kill(getpid(), sig));
}
#endif

#ifdef RT_SOLARIS
#define SA_HANDLER(f) ((void (*)(...))(f))
#endif

#ifdef RT_HPUX
#define SA_HANDLER(f) ((void (*)(...))(f))
#endif

#ifdef RT_CONVEX
#define SA_HANDLER(f) ((void (*)(...))(f))
#endif

// ----------------------------------------------------------------------------
//
// timing
//
// ----------------------------------------------------------------------------

#if defined(RT_SUNOS)
#include <sys/timeb.h>
extern "C" { int ftime(struct timeb *); };
static inline int real_time(void) 
{ 
  timeb tb;
  ftime(&tb);
  return(tb.time*1000+tb.millitm);
}
#else
static inline int real_time(void) 
{ 
  struct tms b; 
  return(times(&b)); 
}
#endif

static inline int real_clock(void) 
{ 
  struct tms b;
  times(&b);
  return(b.tms_utime+b.tms_stime); 
}

static inline int real_all(void) 
{ 
  struct tms b;
  times(&b);
  return(b.tms_utime+b.tms_stime+b.tms_cutime+b.tms_cstime);
}

#ifdef RT_LINUX
static inline int ms_time(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_clock(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_all(int t) { return(((t)*1000)/CLK_TCK); }
#endif

#ifdef RT_PTX
static inline int ms_time(int t) { return(((t)*1000)/HZ); }
static inline int ms_clock(int t) { return(((t)*1000)/HZ); }
static inline int ms_all(int t) { return(((t)*1000)/HZ); }
#endif

#ifdef RT_IRIX
static inline int ms_time(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_clock(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_all(int t) { return(((t)*1000)/CLK_TCK); }
#endif

#ifdef RT_SUNOS
static inline int ms_time(int t) { return(t); }
static inline int ms_clock(int t) { return(((t)*1000)/HZ); }
static inline int ms_all(int t) { return(((t)*1000)/HZ); }
#endif

#ifdef RT_SOLARIS
extern "C" { long _sysconf(int); }
static inline int ms_time(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_clock(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_all(int t) { return(((t)*1000)/CLK_TCK); }
#endif

#ifdef RT_HPUX
static inline int ms_time(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_clock(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_all(int t) { return(((t)*1000)/CLK_TCK); }
#endif

#ifdef RT_CONVEX
static inline int ms_time(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_clock(int t) { return(((t)*1000)/CLK_TCK); }
static inline int ms_all(int t) { return(((t)*1000)/CLK_TCK); }
#endif

// ----------------------------------------------------------------------------
//
// stack direction (RT_GROW_UP or RT_GROW_DOWN)
//
// ----------------------------------------------------------------------------

#ifdef RT_i386
#define RT_GROW_DOWN
#endif

#ifdef RT_MIPS
#define RT_GROW_DOWN
#endif        

#ifdef RT_MIPS2
#define RT_GROW_DOWN
#endif        

#ifdef RT_SPARC
#define RT_GROW_DOWN
#endif

#ifdef RT_PARISC
#define RT_GROW_UP
#endif

// ----------------------------------------------------------------------------
//
// minimum alignment for all types in bytes
// this MUST be a power of two, otherwise redefine rt_alignment()!!!
//
// ----------------------------------------------------------------------------

#ifdef RT_i386
#define RT_ALIGNMENT (4)
#endif

#ifdef RT_MIPS
#define RT_ALIGNMENT (8)
#endif        

#ifdef RT_MIPS2
#define RT_ALIGNMENT (8)
#endif        

#ifdef RT_SPARC
#define RT_ALIGNMENT (8)
#endif

#ifdef RT_PARISC
#define RT_ALIGNMENT (8)
#endif

// ----------------------------------------------------------------------------
//
// atomic testAndSet operation
// (sets the value of an integer to ONE and return its old value)
//
// ----------------------------------------------------------------------------

#ifdef RT_i386
#ifdef __GNUC__
static inline 
int testAndSet(volatile int *mutex) 
{
  int result;
  asm volatile("movl %2, %0; xchgl %0, (%1)" : "=&r" (result) : "r" (mutex),
	       "i" (1) );
  return result;
}
#else
extern "C" { int uAtomic(volatile int *); }
#define testAndSet(mutex) uAtomic(mutex)
#endif
#endif

#ifdef RT_MIPS
extern "C" { int atomic_test_and_set(volatile int *); }
#define testAndSet(mutex) atomic_test_and_set(mutex)
#endif        

#ifdef RT_MIPS2
extern "C" { int uAtomic(volatile int *); }
#define testAndSet(mutex) uAtomic(mutex)
#endif        

#ifdef RT_SPARC
extern "C" { int uAtomic(volatile int *); }
#define testAndSet(mutex) uAtomic(mutex)
#endif

#ifdef RT_PARISC
// atomically sets mutex to ZERO, mutex must be 16 byte aligned 
// and no other data must be in same 16 byte cache line
// special treatment in rt_mutex required
extern "C" { int testAndSet(volatile int *mutex); }
#endif

#endif
// ----------------------------------------------------------------------------
// end of rt_machine.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
