// ----------------------------------------------------------------------------
//
// rt_pointer.i
// RT++ garbage collected pointer objects
//
// $Id: rt_pointer.i,v 1.2 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_POINTER_I
#define RT_POINTER_I

// ----------------------------------------------------------------------------
//
// return block reference from pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline Block * 
Pointer<T>::block(T *p)
{
  return((Block*)Block::block(p));   
}

// ----------------------------------------------------------------------------
//
// allocate object
// caller is in charge of unlocking processor!!!
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
Pointer<T>::_alloc(void)
{
  T *p = (T*)processor.alloc(sizeof(T));
  pointer = p;
  void (*const f)(T *) = Type<T>::init;
  if (f) f(p);     
}

// ----------------------------------------------------------------------------
//
// nil initialization
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
Pointer<T>::init(void)
{
  pointer = 0;
}

// ----------------------------------------------------------------------------
//
// allocate object
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
Pointer<T>::alloc(void)
{
  _alloc();
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by nil pointer
//
// ----------------------------------------------------------------------------
template <class T>
inline
Pointer<T>::Pointer(void)
{
  init();
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// initialization by heap object
//
// ----------------------------------------------------------------------------
template <class T>
inline
Pointer<T>::Pointer(int n)
{
  if (n != 1) rt_abort("only Pointer(1) initialization allowed");
  _alloc();
  processor.push(this, (void(*)(void*))scan);
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// initialization by pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline
Pointer<T>::Pointer(const Pointer<T> &p)
{
  pointer = p.pointer;
  processor.push(this, (void(*)(void*))scan);
}

// ----------------------------------------------------------------------------
//
// destroy pointer
//
// ----------------------------------------------------------------------------
template <class T>
inline
Pointer<T>::~Pointer(void)
{
  processor.pop(&pointer);
}

// ----------------------------------------------------------------------------
//
// assignment of pointer <p>
//
// ----------------------------------------------------------------------------
template <class T>
inline Pointer<T>& 
Pointer<T>::operator = (const Pointer<T> &p)
{
  pointer = p.pointer;
  return(*this);
}

// ----------------------------------------------------------------------------
//
// equality of pointers
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Pointer<T>::operator == (const Pointer<T> &p)
{
  return(pointer == p.pointer);
}

// ----------------------------------------------------------------------------
//
// test for nil pointer (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline int 
Null(const Pointer<T> &p)
{
  return(p.pointer == 0);
}

// ----------------------------------------------------------------------------
//
// allocate object (static member function)
//
// ----------------------------------------------------------------------------
template <class T>
inline Pointer<T> 
Pointer<T>::Alloc(void)
{
  Pointer<T> p(1);
  return(p);
}

// ----------------------------------------------------------------------------
//
// dereferencing of pointers, returns an alias!
//
// ----------------------------------------------------------------------------
template <class T>
inline T& 
Pointer<T>::operator * (void)
{
  return(*pointer);
}

// ----------------------------------------------------------------------------
//
// dereferencing of pointers, returns an address!
//
// ----------------------------------------------------------------------------
template <class T>
inline T* 
Pointer<T>::operator -> (void)
{
  return(pointer);
}

// ----------------------------------------------------------------------------
//
// print contents of pointer (friend function)
//
// ----------------------------------------------------------------------------
template <class T>
inline ostream& 
operator << (ostream& s, const Pointer<T>& p)
{
  s << p.pointer;
  return s;
}

// ----------------------------------------------------------------------------
//
// scan referenced object
//
// ----------------------------------------------------------------------------
template <class T>
inline void 
Pointer<T>::scan(void)
{
  T *p = pointer;
  if (!p) return;               
  if (block(p)->mark()) return;
  void (*const f)(T *) = Type<T>::scan; 
  if (f) f(p);
}

#endif
// ----------------------------------------------------------------------------
// end of rt_pointer.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
