// ----------------------------------------------------------------------------
//
// rt_result.i
// RT++ result threads (typed in result)
//
// $Id: rt_result.i,v 1.3 1996/03/13 19:33:29 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_RESULT_I
#define RT_RESULT_I

// ----------------------------------------------------------------------------
//
// scan function for this type
//
// ----------------------------------------------------------------------------
template <class R>
inline void 
ResultThread<R>::_scan(void)
{
  CoreThread::scan();
  void(*const f)(R*) = Type<R>::scan;
  if (f) f(&result);
}

// ----------------------------------------------------------------------------
//
// scan function called by garbage collection
//
// ----------------------------------------------------------------------------
template <class R>
inline void 
ResultThread<R>::scan(void)
{
  scanfun(this);
}

// ----------------------------------------------------------------------------
//
// create result thread
//
// ----------------------------------------------------------------------------
template <class R>
inline void 
ResultThread<R>::init(void)
{
  CoreThread::init();
  void (*f)(R*) = Type<R>::init;
  if (f) f(&result);
  scanfun = (void(*)(CoreThread*))ResultThread<R>::scan;
}

// ----------------------------------------------------------------------------
//
// create result thread
//
// ----------------------------------------------------------------------------
template <class R>
inline 
ResultThread<R>::ResultThread(void)
{
  init();
}

// ----------------------------------------------------------------------------
//
// create result thread (lazy thread creation)
//
// ----------------------------------------------------------------------------
template <class R>
inline void 
ResultThread<R>::create(void)
{
  sys->is_lazy(this);   // place thread into lazy queue
}

// ----------------------------------------------------------------------------
//
// start result thread (eager thread creation)
//
// ----------------------------------------------------------------------------
template <class R>
inline void 
ResultThread<R>::start(void)
{
  realize();           // allocate stack
  sys->is_ready(this); // place thread into ready queue
}

// ----------------------------------------------------------------------------
//
// return result value of thread
//
// ----------------------------------------------------------------------------
template <class R>
inline R 
ResultThread<R>::wait(void) 
{
  if (!avail) CoreThread::wait();
  return(result);
}

#endif
// ----------------------------------------------------------------------------
// end of result.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
