// ----------------------------------------------------------------------------
//
// rt_tptr.i
// RT++ pointer to threads (garbage collected)
//
// $Id: rt_tptr.i,v 1.7 1996/03/22 08:46:09 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_TPTR_I
#define RT_TPTR_I

// ----------------------------------------------------------------------------
//
// thread pointer typed in result only
//
// ----------------------------------------------------------------------------

template <class R>
inline 
Thread<R>::Thread(void): Pointer< ResultThread<R> >()
{
}

// initialization to simulate subtyping (see conversion operators below)
template <class R>
inline 
Thread<R>::Thread(ResultThread<R> *p)
{
  pointer = p;
}

template <class R>
inline R 
Wait(const Thread<R> &t)
{
  return(t.pointer->wait());
}

// ----------------------------------------------------------------------------
//
// thread pointer with no argument
//
// ----------------------------------------------------------------------------

template <class R>
inline void 
Thread0<R>::_init(R (*f)(void))
{
  pointer = (ArgumentThread0<R>*)
    processor.alloc(sizeof(ArgumentThread0<R>));
  pointer->init(f);
}

template <class R>
inline void 
Thread0<R>::_create(R (*f)(void))
{
  _init(f);
#ifndef RT_THREAD_NOTPAR
  pointer->create();
#endif
}

template <class R>
inline void 
Thread0<R>::_start(R (*f)(void))
{
  _init(f);
#ifndef RT_THREAD_NOTPAR
  pointer->start();
#endif
}

template <class R>
inline void 
Thread0<R>::create(R (*f)(void))
{
  _create(f);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R>
inline void 
Thread0<R>::start(R (*f)(void))
{
  _start(f);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R>
inline 
Thread0<R>::Thread0(void): Pointer< ArgumentThread0<R> >()
{
}

template <class R>
inline 
Thread0<R>::Thread0(R (*f)(void))
{
#ifdef RT_THREAD_NOTLAZY
  _start(f);
#else
  _create(f);
#endif
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R>
inline Thread0<R>
Create(R (*f)(void))
{
  Thread0<R> t;
  t.create(f);
  return(t);
}

template <class R>
inline Thread0<R>
Start(R (*f)(void))
{
  Thread0<R> t;
  t.start(f);
  return(t);
}

template <class R>
inline R 
Wait(const Thread0<R> &t)
{
  return(t.pointer->wait());
}

template <class R>
inline 
Thread0<R>::operator Thread<R>()
{
  return(Thread<R>((ResultThread<R>*)pointer));
}

// ----------------------------------------------------------------------------
//
// thread pointer with one argument
//
// ----------------------------------------------------------------------------

template <class R, class A1>
inline void
Thread1<R, A1>::_init(R (*f)(A1), A1 a1)
{
  pointer = (ArgumentThread1<R, A1>*)
    processor.alloc(sizeof(ArgumentThread1<R, A1>));
  pointer->init(f, a1);
}

template <class R, class A1>
inline void
Thread1<R, A1>::_create(R (*f)(A1), A1 a1)
{
  _init(f, a1);
#ifndef RT_THREAD_NOTPAR
  pointer->create();
#endif
}

template <class R, class A1>
inline void 
Thread1<R, A1>::_start(R (*f)(A1), A1 a1)
{
  _init(f, a1);
#ifndef RT_THREAD_NOTPAR
  pointer->start();
#endif
}

template <class R, class A1>
inline void 
Thread1<R, A1>::create(R (*f)(A1), A1 a1)
{
  _create(f, a1);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1>
inline void 
Thread1<R, A1>::start(R (*f)(A1), A1 a1)
{
  _start(f, a1);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1>
inline 
Thread1<R, A1>::Thread1(void): Pointer< ArgumentThread1<R, A1> >()
{
}

template <class R, class A1>
inline 
Thread1<R, A1>::Thread1(R (*f)(A1), A1 a1)
{
#ifdef RT_THREAD_NOTLAZY
  _start(f, a1);
#else
  _create(f, a1);
#endif
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1>
inline Thread1<R, A1>
Create(R (*f)(A1), A1 a1)
{
  Thread1<R, A1> t;
  t.create(f, a1);
  return(t);
}

template <class R, class A1>
inline Thread1<R, A1>
Start(R (*f)(A1), A1 a1)
{
  Thread1<R, A1> t;
  t.start(f, a1);
  return(t);
}

template <class R, class A1>
inline R 
Wait(const Thread1<R, A1> &t)
{
  return(t.pointer->wait());
}

template <class R, class A1>
inline 
Thread1<R, A1>::operator Thread<R>()
{
  return(Thread<R>((ResultThread<R>*)pointer));
}

// ----------------------------------------------------------------------------
//
// thread pointer with two arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2>
inline void 
Thread2<R, A1, A2>::_init(R (*f)(A1, A2), A1 a1, A2 a2)
{
  pointer = (ArgumentThread2<R, A1, A2>*)
    processor.alloc(sizeof(ArgumentThread2<R, A1, A2>));
  pointer->init(f, a1, a2);
}

template <class R, class A1, class A2>
inline void 
Thread2<R, A1, A2>::_create(R (*f)(A1, A2), A1 a1, A2 a2)
{
  _init(f, a1, a2);
#ifndef RT_THREAD_NOTPAR
  pointer->create();
#endif
}

template <class R, class A1, class A2>
inline void 
Thread2<R, A1, A2>::_start(R (*f)(A1, A2), A1 a1, A2 a2)
{
  _init(f, a1, a2);
#ifndef RT_THREAD_NOTPAR
  pointer->start();
#endif
}

template <class R, class A1, class A2>
inline void 
Thread2<R, A1, A2>::create(R (*f)(A1, A2), A1 a1, A2 a2)
{
  _create(f, a1, a2);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2>
inline void 
Thread2<R, A1, A2>::start(R (*f)(A1, A2), A1 a1, A2 a2)
{
  _start(f, a1, a2);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2>
inline 
Thread2<R, A1, A2>::Thread2(void): Pointer< ArgumentThread2<R, A1, A2> >()
{
}

template <class R, class A1, class A2>
inline 
Thread2<R, A1, A2>::Thread2(R (*f)(A1, A2), A1 a1, A2 a2)
{
#ifdef RT_THREAD_NOTLAZY
  _start(f, a1, a2);
#else
  _create(f, a1, a2);
#endif
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2>
inline Thread2<R, A1, A2>
Create(R (*f)(A1, A2), A1 a1, A2 a2)
{
  Thread2<R, A1, A2> t;
  t.create(f, a1, a2);
  return(t);
}

template <class R, class A1, class A2>
inline Thread2<R, A1, A2>
Start(R (*f)(A1, A2), A1 a1, A2 a2)
{
  Thread2<R, A1, A2> t;
  t.start(f, a1, a2);
  return(t);
}

template <class R, class A1, class A2>
inline R 
Wait(const Thread2<R, A1, A2> &t)
{
  return(t.pointer->wait());
}

template <class R, class A1, class A2>
inline 
Thread2<R, A1, A2>::operator Thread<R>()
{
  return(Thread<R>((ResultThread<R>*)pointer));
}

// ----------------------------------------------------------------------------
//
// thread pointer with three arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2, class A3>
inline void 
Thread3<R, A1, A2, A3>::_init(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  pointer = (ArgumentThread3<R, A1, A2, A3>*)
    processor.alloc(sizeof(ArgumentThread3<R, A1, A2, A3>));
  pointer->init(f, a1, a2, a3);
}

template <class R, class A1, class A2, class A3>
inline void 
Thread3<R, A1, A2, A3>::_create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  _init(f, a1, a2, a3);
#ifndef RT_THREAD_NOTPAR
  pointer->create();
#endif
}

template <class R, class A1, class A2, class A3>
inline void 
Thread3<R, A1, A2, A3>::_start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  _init(f, a1, a2, a3);
#ifndef RT_THREAD_NOTPAR
  pointer->start();
#endif
}

template <class R, class A1, class A2, class A3>
inline void 
Thread3<R, A1, A2, A3>::create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  _create(f, a1, a2, a3);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3>
inline void 
Thread3<R, A1, A2, A3>::start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  _start(f, a1, a2, a3);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3>
inline 
Thread3<R, A1, A2, A3>::
Thread3(void): Pointer< ArgumentThread3<R, A1, A2, A3> >()
{
}

template <class R, class A1, class A2, class A3>
inline 
Thread3<R, A1, A2, A3>::Thread3(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
#ifdef RT_THREAD_NOTLAZY
  _start(f, a1, a2, a3);
#else
  _create(f, a1, a2, a3);
#endif
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3>
inline Thread3<R, A1, A2, A3>
Create(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  Thread3<R, A1, A2, A3> t;
  t.create(f, a1, a2, a3);
  return(t);
}

template <class R, class A1, class A2, class A3>
inline Thread3<R, A1, A2, A3>
Start(R (*f)(A1, A2, A3), A1 a1, A2 a2, A3 a3)
{
  Thread3<R, A1, A2, A3> t;
  t.start(f, a1, a2, a3);
  return(t);
}

template <class R, class A1, class A2, class A3>
inline R 
Wait(const Thread3<R, A1, A2, A3> &t)
{
  return(t.pointer->wait());
}

template <class R, class A1, class A2, class A3>
inline 
Thread3<R, A1, A2, A3>::operator Thread<R>()
{
  return(Thread<R>((ResultThread<R>*)pointer));
}

// ----------------------------------------------------------------------------
//
// thread pointer with four arguments
//
// ----------------------------------------------------------------------------

template <class R, class A1, class A2, class A3, class A4>
inline void 
Thread4<R, A1, A2, A3, A4>::
_init(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  pointer = (ArgumentThread4<R, A1, A2, A3, A4>*)
    processor.alloc(sizeof(ArgumentThread4<R, A1, A2, A3, A4>));
  pointer->init(f, a1, a2, a3, a4);
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
Thread4<R, A1, A2, A3, A4>::
_create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  _init(f, a1, a2, a3, a4);
#ifndef RT_THREAD_NOTPAR
  pointer->create();
#endif
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
Thread4<R, A1, A2, A3, A4>::
_start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  _init(f, a1, a2, a3, a4);
#ifndef RT_THREAD_NOTPAR
  pointer->start();
#endif
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
Thread4<R, A1, A2, A3, A4>::
create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  _create(f, a1, a2, a3, a4);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3, class A4>
inline void 
Thread4<R, A1, A2, A3, A4>::
start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  _start(f, a1, a2, a3, a4);
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3, class A4>
inline 
Thread4<R, A1, A2, A3, A4>::
Thread4(void): Pointer< ArgumentThread4<R, A1, A2, A3, A4> >()
{
}

template <class R, class A1, class A2, class A3, class A4>
inline 
Thread4<R, A1, A2, A3, A4>::
Thread4(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
#ifdef RT_THREAD_NOTLAZY
  _start(f, a1, a2, a3, a4);
#else
  _create(f, a1, a2, a3, a4);
#endif
  processor.unlock();
#ifdef RT_THREAD_NOTPAR
  pointer->user();
#endif
}

template <class R, class A1, class A2, class A3, class A4>
inline Thread4<R, A1, A2, A3, A4>
Create(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  Thread4<R, A1, A2, A3, A4> t;
  t.create(f, a1, a2, a3, a4);
  return(t);
}

template <class R, class A1, class A2, class A3, class A4>
inline Thread4<R, A1, A2, A3, A4>
Start(R (*f)(A1, A2, A3, A4), A1 a1, A2 a2, A3 a3, A4 a4)
{
  Thread4<R, A1, A2, A3, A4> t;
  t.start(f, a1, a2, a3, a4);
  return(t);
}

template <class R, class A1, class A2, class A3, class A4>
inline R 
Wait(const Thread4<R, A1, A2, A3, A4> &t)
{
  return(t.pointer->wait());
}

template <class R, class A1, class A2, class A3, class A4>
inline 
Thread4<R, A1, A2, A3, A4>::operator Thread<R>()
{
  return(Thread<R>((ResultThread<R>*)pointer));
}

#endif
// ----------------------------------------------------------------------------
// end of rt_tptr.i
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
