// ----------------------------------------------------------------------------
//
// rt_type.h
// RT++ type information for garbage collection
//
// $Id: rt_type.h,v 1.4 1996/02/28 15:51:55 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ----------------------------------------------------------------------------
#ifndef RT_TYPE_H
#define RT_TYPE_H

// ----------------------------------------------------------------------------
// 
// type information for memory management and garbage collection
// the init routine is used for initialization of allocated objects
// the scan routine is used for the marking phase of the garbage collector
//
// ----------------------------------------------------------------------------
template <class T>
class Type
{
  public:
  static void (*const init)(T *); // initialize all pointers to 0
  static void (*const scan)(T *); // mark all referenced shared memory blocks
};

// ----------------------------------------------------------------------------
// 
// initialization macros
//
// ----------------------------------------------------------------------------

// this is ANSI C++ but not implemented in the GNU compiler yet
// it should represent the default values of the Type<T> components

// template <class T>
// void (*const Type<T>::scan)(T *) = (void(*const)(T*))T::scan;
// template <class T>
// void (*const Type<T>::init)(T *) = (void(*const)(T*))T::init;

// use this for class types that define scan and init
#define Ref(T) \
  void (*const Type<T>::scan)(T *) = (void(*const)(T*))T::scan; \
  void (*const Type<T>::init)(T *) = (void(*const)(T*))T::init

// use this for atomic non-pointer types
#define Atom(T) \
  void (*const Type<T>::scan)(T *) = (void(*const)(T*))0; \
  void (*const Type<T>::init)(T *) = (void(*const)(T*))0

// use this for each thread result type
#define ThreadRes(R) Ref(ResultThread<R>)

// use this for each thread type
#define ThreadArg0(N, R) \
   typedef ArgumentThread0<R> N; \
   Ref(N)
#define ThreadArg1(N, R, A1) \
   typedef ArgumentThread1<R, A1> N; \
   Ref(N)
#define ThreadArg2(N, R, A1, A2) \
   typedef ArgumentThread2<R, A1, A2> N; \
   Ref(N)
#define ThreadArg3(N, R, A1, A2, A3) \
   typedef ArgumentThread3<R, A1, A2, A3> N; \
   Ref(N)
#define ThreadArg4(N, R, A1, A2, A3, A4) \
   typedef ArgumentThread4<R, A1, A2, A3, A4> N; \
   Ref(N)

// use this for each bag result type
#define ThreadBagRes(R) Ref(ThreadBagBase<R>)

#endif
// ----------------------------------------------------------------------------
// end of rt_type.h
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
