// ----------------------------------------------------------------------------
//
// rt_lib.cc
// RT++ library functions
//
// $Id: rt_lib.cc,v 1.6 1996/03/13 12:25:48 ws Exp $
// (C) 1996 Wolfgang Schreiner <Wolfgang.Schreiner@risc.uni-linz.ac.at>
//
// ---------------------------------------------------------------------------

#include "rt++.h"
 
// ----------------------------------------------------------------------------
//
// bind thread to processor <n>
//
// ----------------------------------------------------------------------------
void rt_bind(int n)
{
  processor.get_thread()->bind(n);
  if (processor.number() != n) rt_yield();
}

// ----------------------------------------------------------------------------
//
// bind thread to current processor
//
// ----------------------------------------------------------------------------
void rt_bind(void)
{
  processor.get_thread()->bind(processor.number());
}

// ----------------------------------------------------------------------------
//
// release processor binding
//
// ----------------------------------------------------------------------------
void rt_unbind(void)
{
  processor.get_thread()->bind(-1);
}

// ----------------------------------------------------------------------------
//
// deschedule thread
//
// ----------------------------------------------------------------------------
void rt_yield(void)
{
  processor.lock();
  CoreThread *next = sys->get_ready();   // get ready thread
  if (next)                              // thread was found
    {					 
      processor.save();                  // going to switch thread
      next->exec_ready();	         // non-local jump and unlock
    }					 
  else					 
    {					 
      processor.unlock();                // exit kernel mode
    }
}

// ----------------------------------------------------------------------------
//
// lock processor (prevent preemption)
//
// ----------------------------------------------------------------------------
void rt_lock(void)
{
  processor.lock();
}

// ----------------------------------------------------------------------------
//
// unlock processor
//
// ----------------------------------------------------------------------------
void rt_unlock(void)
{
  processor.unlock();
}

// ----------------------------------------------------------------------------
//
// return time in ms
//
// ----------------------------------------------------------------------------
int rt_clock(void)
{
  return(ms_time(real_time()));
}

// ----------------------------------------------------------------------------
//
// trigger garbage collection
//
// ----------------------------------------------------------------------------
void rt_gc(void)
{
  heap->collect();
}

// ----------------------------------------------------------------------------
//
// set <a> to program argument
//
// ----------------------------------------------------------------------------
void rt_arg(RT_Argument *a)
{
  *a = rt_argument;
}

// ----------------------------------------------------------------------------
//
// return number of processors
//
// ----------------------------------------------------------------------------
int rt_proc(void)
{
  return(rt_argument.proc);
}

// ----------------------------------------------------------------------------
// end of rt_lib.cc
// ----------------------------------------------------------------------------
// Local Variables: ***
// mode: C++ ***
// End: ***
